import { RemovalPolicy } from 'aws-cdk-lib';
import { IRole, ISamlProvider, IUser } from 'aws-cdk-lib/aws-iam';
import { CfnDataLakeSettings, CfnPrincipalPermissions, CfnResource } from 'aws-cdk-lib/aws-lakeformation';
import { IBucket } from 'aws-cdk-lib/aws-s3';
import { AwsCustomResource } from 'aws-cdk-lib/custom-resources';
import { Construct } from 'constructs';
import { PermissionModel } from '../../utils';
/**
 * Configure the DataLakeSettings of LakeFormation
 * @param scope the construct scope
 * @param id the construct id
 * @param principals the list of principals to add as LakeFormation admin
 * @returns the CfnDataLakeSettings to configure Lake Formation
 */
export declare function putDataLakeSettings(scope: Construct, id: string, principals: (IRole | IUser | [ISamlProvider, string])[]): CfnDataLakeSettings;
/**
 * Register an Amazon S3 location in AWS Lake Formation.
 * It creates an IAM Role dedicated per location and register the location using either Lake Formation or Hybrid access model.
 * @param scope the construct scope
 * @param id the construct id
 * @param locationBucket the Amazon S3 location bucket
 * @param locationPrefix the Amazon S3 location prefix
 * @param accessMode the Amazon S3 location access model
 * @return the CfnDataLakeSettings to register the Amazon S3 location in AWS Lake Formation
 */
export declare function registerS3Location(scope: Construct, id: string, locationBucket: IBucket, locationPrefix?: string, accessMode?: PermissionModel, dataAccessRole?: IRole): [IRole, CfnResource];
/**
 * Revoke the IAMAllowedPrincipal permissions from the database.
 * @param scope the construct scope
 * @param id the construct id
 * @param database the database to remove the IAMAllowedPrincipal permission
 * @return the CfnDataLakeSettings to remove the IAMAllowedPrincipal permission
 */
export declare function revokeIamAllowedPrincipal(scope: Construct, id: string, database: string, execRole: IRole, removalPolicy: RemovalPolicy): AwsCustomResource;
/**
 * Grant Lake Formation access on Data Lake Location
 * @param scope the construct scope
 * @param id the construct id
 * @param location the Amazon S3 location in ARN format
 * @param principal the IAM Principal to grant Lake Formation access on Data Lake Location
 * @param grantable whether the grantable permission is set. @default - false
 * @return the CfnPermissions to grant Lake Formation access on Data Lake Location
 */
export declare function grantDataLakeLocation(scope: Construct, id: string, location: string, principal: IRole, grantable?: boolean): CfnPrincipalPermissions;
/**
 * Grant Lake Formation permissions required by crawlers
 * @param scope the construct scope
 * @param id the construct id
 * @param database the database to grant Lake Formation permissions
 * @param principal the IAM Principal to grant Lake Formation permissions
 * @return the CfnPrincipalPermissions granting Lake Formation permissions
 */
export declare function grantCrawler(scope: Construct, id: string, database: string, principal: IRole): [CfnPrincipalPermissions, CfnPrincipalPermissions];
